;  BLINE.LSP  -- by JPBeal, 23 July 90
;    AutoCAD Rel 10 AutoLISP routine to create parallel break lines offset
;    from two user supplied points.
;    Variable o1 contains the offset, assigned in function C:BLINE
;    (load) .LSP file into AutoCAD, issue BLINE command.  Object snap
;    modes and point filters may be used.

(defun bl (p1 p2 / p2 echo blip d1 d2 d3 d4 d5 d6 d7 s1 a1 a2 a3)
      (setvar "CMDECHO" 0)                          ;turn command echo off
      (setvar "BLIPMODE" 0)                         ;turn blipmode off
      (setq d1 (distance p1 p2)                     ;total line length
            d2 (* 2.0 (getvar "LTSCALE"))           ;segment repeating length
            d3 (/ d2 24.0)                          ;brk. segment length
            d4 (* d3 (sqrt 2.0))                    ;brk. segment unit length
            d6 (- d2 (* 6.0 d3))                    ;trailer segment length
            s1 (fix (/ d1 d2))                      ;number of segments
            d7 (/ (- d1 (* d2 s1)) 2.0)             ;start and end extended segment
            a1 (angle p1 p2)                        ;direction angle
            a2 (+ a1 (/ pi 4.0))                    ;start brk. segment angle
            a3 (+ a2 (* pi 1.5))                    ;middle brk. segment angle
      )
      (command ".pline")                            ;begin PolyLINE command
      (command p1)                                  ;start point
                                                    ;PLINE started in c:bline()
      (setq p1 (polar p1 a1 (+ d7 (/ d6 2.0))))     ;reset point pointer
      (while (> s1 0)                               ;track number of segments
        (command p1)                                ;draw to start of brk.
        (setq p1 (polar p1 a2 d4))                  ;reset point pointer
        (command p1)                                ;draw first segment of brk.
        (setq p1 (polar p1 a3 (* 3.0 d4)))          ;reset point pointer
        (command p1)                                ;draw middle segment of brk.
        (setq p1 (polar p1 a2 (* 2.0 d4)))          ;reset point pointer
        (command p1)                                ;draw last segment of brk.
        (setq p1 (polar p1 a1 d6)                   ;reset point pointer
              s1 (1- s1)                            ;update counter
        )
      )
      (command p2)                                  ;draw to endpoint
      (command)                                     ;close PolyLINE command
      (setvar "CMDECHO" bl_echo)                    ;restore command echo
      (setvar "BLIPMODE" bl_blip)                   ;restore blipmode
  (eval 'p2)                                        ;return endpoint
)

(defun c:bline (/ p1 pz2 a1)
  (setq *old_err* *error*                           ;store existing error routine
        *error*   *bl_err*                          ;assign new error routine
        bl_echo   (getvar "CMDECHO")                ;store existing CMDECHO
        bl_blip   (getvar "BLIPMODE")               ;store existing BLIPMODE
        p_2       (/ pi 2.0)                        ;90 degrees (PI/2)
        o1        (* 0.125 (getvar "LTSCALE"))      ;offset distance
  )
  (setq p1 (getpoint "\nFrom point: "))             ;get start point
  (if (not (= nil p1))                              ;check for valid input
    (progn
      (setq pz2 (getpoint p1 "\nTo point: "))       ;get end point
      (if (not (= nil pz2))                         ;check for valid input
        (progn
          (setq a1 (angle p1 pz2))
          (bl (polar p1 (- a1 p_2) o1)              ;brk line 1, pt1
              (polar pz2 (- a1 p_2) o1))            ;  brk line 1, pt2
          (bl (polar pz2 (+ a1 p_2) o1)             ;brk line 2, pt1
              (polar p1 (+ a1 p_2) o1))             ;  brk line 2, pt2
        )
      )
    )
  )
  (setq *error*   *old_err*                         ;restore error routine
        *old_err* nil                               ;delete variable
        bl_echo   nil                               ;delete variable
        bl_blip   nil                               ;delete variable
        o1        nil                               ;delete variable
  )
  (princ)                                           ;prevents garbage at termination
)

